package com.shcd.util;

import java.net.NetworkInterface;
import java.util.Collections;
import java.util.List;
import java.util.Scanner;

/**
 * @author 上海城地
 * @description 获取本机相关地址工具类
 * @create 2024-04-13 15:55
 * @since 1.0.0
 */
public class MachineAddrUtil {

    /**
     * 获取网卡物理地址工具类
     *
     * @return
     */
    public static String getMac() {
        String result = null;
        try {
            // 获取所有网络接口
            List<NetworkInterface> networkInterfaces = Collections.list(NetworkInterface.getNetworkInterfaces());
            for (NetworkInterface networkInterface : networkInterfaces) {
                // 忽略回环接口
                if (networkInterface.isLoopback() || !networkInterface.isUp()) {
                    continue;
                }

                byte[] macByte = networkInterface.getHardwareAddress();
                if (macByte != null) {
                    StringBuilder sb = new StringBuilder();
                    for (int i = 0; i < macByte.length; i++) {
                        sb.append(String.format("%02X%s", macByte[i], (i < macByte.length - 1) ? "-" : ""));
                    }
                    result = sb.toString();
                    break; // 找到有效的MAC地址后退出循环
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 获取 CPU 序列号
     *
     * @return
     */
    public static String getCPUSerial() {
        String cpuSerial = System.getenv("PROCESSOR_IDENTIFIER");
        return cpuSerial;
    }

    /**
     * 获取主板序列号
     *
     * @return
     */
    public static String getMainBoardSerial() {
        String result = null;
        try {
            Process process = Runtime.getRuntime().exec(new String[]{"wmic", "baseboard", "get", "serialnumber"});
            process.getOutputStream().close();
            Scanner scanner = new Scanner(process.getInputStream());
            while (scanner.hasNext()) {
                result = scanner.next();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 测试
     * @param args
     */
    public static void main(String[] args) {
        System.out.println("本机的【网卡物理地址】=" + getMac());
        System.out.println("本机的【CPU 序列号】=" + getCPUSerial());
        System.out.println("本机的【主板序列号】=" + getMainBoardSerial());
    }

}
